namespace RenderersLibrary.MultipleGeneratedItems
{
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Kodeo.Reegenerator.Generators;

    /// <summary>
    /// MultipleGeneratedItems custom generator class.
    /// </summary>
    [TemplateDisplay(DisplayName = "MultipleGeneratedItems", Description = "MultipleGeneratedItems")]
    [DefaultExtension(Extension = "txt")]
    public class MultipleGeneratedItems : CodeRenderer
    {
        /// <summary>
        /// Renders the code.
        /// </summary>
        /// <returns>A <see cref="RenderResults"/> that contains the content of the generated files.</returns>
        public override RenderResults Render()
        {
            StringBuilder sb = new StringBuilder();
            List<AdditionalFile> ar = new List<AdditionalFile>();

            foreach (var pair in base.ProjectItem.GetClasses())
            {
                sb.AppendLine(pair.Key);

                var properties = (from i in pair.Value                                                // for all class's partial definitions
                                  from codeElement in i.Members.Cast<EnvDTE.CodeElement>()            // for all code elements in the partial definitions
                                  where codeElement.Kind == EnvDTE.vsCMElement.vsCMElementProperty    // that are properties
                                  select codeElement).Cast<EnvDTE.CodeProperty>().ToList();           // select them as a list of CodeFunction
                foreach (var prop in properties)
                {
                    sb.AppendFormat("\t{0}\r\n", prop.Name);
                    ar.Add(new AdditionalFile(prop.Name + ".txt", "Generated code for property " + prop.Name));
                }
            }

            RenderResults renderResults = new RenderResults(sb.ToString());
            renderResults.AdditionalResults.AddRange(ar);
            return renderResults;
        }
    }
}